//
//  File:  GpioSample.h
//
#ifndef __GPIOSAMPLE_H
#define __GPIOSAMPLE_H

#ifdef __cplusplus
extern "C" {
#endif

//------------------------------------------------------------------------------
//
//  Define:  GPIO_DEVICE_NAME
//
#define GPIO_DEVICE_NAME        L"GIO1:"

//------------------------------------------------------------------------------
//
//  Define:  GPIO_DIR_xxx/GPIO_INT_xxx
//
#define GPIO_DIR_OUTPUT         (0 << 0)
#define GPIO_DIR_INPUT          (1 << 0)



#define IOCTL_GPIO_SETBIT       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0300, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_CLRBIT       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0301, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_GETBIT       \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0303, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_SETMODE      \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0304, METHOD_BUFFERED, FILE_ANY_ACCESS)

#define IOCTL_GPIO_GETMODE      \
    CTL_CODE(FILE_DEVICE_UNKNOWN, 0x0305, METHOD_BUFFERED, FILE_ANY_ACCESS)



__inline HANDLE GPIOOpen()
{
    HANDLE hDevice = INVALID_HANDLE_VALUE;

    hDevice = CreateFile(GPIO_DEVICE_NAME, 0, 0, NULL, 0, 0, NULL);
    if (hDevice == INVALID_HANDLE_VALUE) 
	    {
        DEBUGMSG(1, (L"GPIOOpen CreateFile failed\r\n"));
	    return NULL;
        }

    return hDevice;
}

__inline void GPIOClose(HANDLE hGPIO)
{
    if (hGPIO != NULL)
    {
        CloseHandle(hGPIO);
    }
}



__inline void GPIOSetAsOutput(HANDLE hGPIO,DWORD num, bool isOutputHigh)
{
	DWORD   dwValue[2];

    dwValue[0] = num;
    dwValue[1] = GPIO_DIR_OUTPUT;

    DeviceIoControl(hGPIO,
					IOCTL_GPIO_SETMODE,
					&dwValue,
					sizeof(dwValue),
					NULL,
					0,
					NULL,
					NULL );
	if(isOutputHigh)
		DeviceIoControl(hGPIO,
                        IOCTL_GPIO_SETBIT,
                        &num,
                        sizeof(num),
                        NULL,
                        0,
                        NULL,
                        NULL );
	else
		DeviceIoControl(hGPIO,
                        IOCTL_GPIO_CLRBIT,
                        &num,
                        sizeof(num),
                        NULL,
                        0,
                        NULL,
                        NULL );	
}

__inline void GPIOSetAsInput(HANDLE hGPIO,DWORD num)
{
	DWORD   dwValue[2];
	
	dwValue[0] = num;
    dwValue[1] = GPIO_DIR_INPUT;

    DeviceIoControl(hGPIO,
					IOCTL_GPIO_SETMODE,
					&dwValue,
					sizeof(dwValue),
					NULL,
					0,
					NULL,
					NULL );
}

//return
__inline BOOL IsGpioOutput(HANDLE hGPIO,DWORD num)
{
	DWORD mode = 0;

    DeviceIoControl(hGPIO,
                    IOCTL_GPIO_GETMODE,
                    &num,
                    sizeof(num),
                    &mode,
                    sizeof(mode),
                    NULL,
                    NULL );

	return !(mode & GPIO_DIR_INPUT);
}

__inline BOOL IsGpioInput(HANDLE hGPIO, DWORD num)
{	
	return !IsGpioOutput(hGPIO,num);
}

__inline BOOL IsGpioInputHigh(HANDLE hGPIO, DWORD num)
{
	DWORD   dwValue = 0;

	DeviceIoControl(hGPIO,
                    IOCTL_GPIO_GETBIT,
                    &num,
                    sizeof(num),
                    &dwValue,
                    sizeof(dwValue),
                    NULL,
                    NULL );

     return dwValue;
}

__inline BOOL IsGpioInputLow(HANDLE hGPIO, DWORD num)
{
	return !IsGpioInputHigh(hGPIO,num);
}


#ifdef __cplusplus
}
#endif

#endif